package ffp_examples;

/*
 * material.java
 * This program demonstrates the use of the FFP lighting model.
 * Several objects are drawn using different material characteristics.
 * A single light source illuminates the objects.
 */
public class material extends DemoBase {

	public static void main(String[] args) {
		new material().init(EUT_RGB);
	}

	static int[] no_mat = { 0, 0, 0, 0 };
	static int[] mat_ambient = { 116, 116, 116 };
	static int[] mat_ambient_color = { 156, 156, 7 };
	static int[] mat_diffuse = { 2, 55, 156 };
	static int[] mat_specular = { 255, 255, 255 };
	static int[] no_shininess = { 0 };
	static int[] low_shininess = { 5 };
	static int[] high_shininess = { 100 };
	static int[] mat_emission = { 18, 7, 7 };

	public static void reshape(int w, int h) {
		ffp.ffpViewport(0, 0, w, h);
		create_projection(w, h);
	}

	private static void create_projection(int w, int h) {
		ffp.ffpMatrixMode(FFP_PROJECTION);
		ffp.ffpLoadIdentity();
		if (w <= (h * 2)) {
			ffp.ffpOrtho(-6.0f, 6.0f, -3.0f * ((float) h * 2) / w, 3.0f * ((float) h * 2) / w, -10.0f, 10.0f);
		} else {
			ffp.ffpOrtho(-6.0f * w / ((float) h * 2), 6.0f * w / ((float) h * 2), -3.0f, 3.0f, -10.0f, 10.0f);
		}
		ffp.ffpMatrixMode(FFP_MODELVIEW);
		ffp.ffpLoadIdentity();
	}

	@Override
	public void dbInit(int width, int height) {
		reshape(width, height);
		ffp.ffpClearColor(0, 2, 2);
		ffp.ffpShadeModel(FFP_SMOOTH);
		ffp.ffpEnable(FFP_DEPTH_TEST);
		float[] ambient = { 0.0f, 0.0f, 0.0f };
		float[] diffuse = { 1.0f, 1.0f, 1.0f };
		float[] position = { 0.0f, 3.0f, 2.0f, 0.0f };
		int[] lmodel_ambient = { 34, 34, 34 };

		ffp.ffpLightfv(0, FFP_AMBIENT, ambient);
		ffp.ffpLightfv(0, FFP_DIFFUSE, diffuse);
		ffp.ffpLightfv(0, FFP_POSITION, position);
		ffp.ffpLightModeliv(FFP_LIGHT_MODEL_AMBIENT, lmodel_ambient);
		ffp.ffpLightModeli(FFP_LIGHT_MODEL_LOCAL_VIEWER, FFP_FALSE);

		ffp.ffpEnable(FFP_LIGHTING);
		ffp.ffpEnableLight(0);
	}

	@Override
	public void dbUpdate() {

	}

	/*
	 *  Draw twelve spheres in 3 rows with 4 columns.  
	 *  The spheres in the first row have materials with no ambient reflection.
	 *  The second row has materials with significant ambient reflection.
	 *  The third row has materials with colored ambient reflection.
	 *
	 *  The first column has materials with blue, diffuse reflection only.
	 *  The second column has blue diffuse reflection, as well as specular
	 *  reflection with a low shininess exponent.
	 *  The third column has blue diffuse reflection, as well as specular
	 *  reflection with a high shininess exponent (a more concentrated highlight).
	 *  The fourth column has materials which also include an emissive component.
	 *
	 *  glTranslatef() is used to move spheres to their appropriate locations.
	 */
	@Override
	public void dbRender() {
		ffp.ffpClear(FFP_VIDEO_BUFFER | FFP_DEPTH_BUFFER);
		/*
		 *  draw sphere in first row, first column
		 *  diffuse reflection only; no ambient or specular
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-3.75f, 3.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, no_mat);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, no_mat);
		ffp.ffpMaterialiv(FFP_SHININESS, no_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, no_mat);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in first row, second column
		 *  diffuse and specular reflection; low shininess; no ambient
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-1.25f, 3.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, no_mat);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialiv(FFP_SHININESS, low_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, no_mat);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in first row, third column
		 *  diffuse and specular reflection; high shininess; no ambient
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(1.25f, 3.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, no_mat);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialiv(FFP_SHININESS, high_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, no_mat);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in first row, fourth column
		 *  diffuse reflection; emission; no ambient or specular reflection
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(3.75f, 3.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, no_mat);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, no_mat);
		ffp.ffpMaterialiv(FFP_SHININESS, no_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, mat_emission);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in second row, first column
		 *  ambient and diffuse reflection; no specular
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-3.75f, 0.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, no_mat);
		ffp.ffpMaterialiv(FFP_SHININESS, no_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, no_mat);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in second row, second column
		 *  ambient, diffuse and specular reflection; low shininess
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-1.25f, 0.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialiv(FFP_SHININESS, low_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, no_mat);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in second row, third column
		 *  ambient, diffuse and specular reflection; high shininess
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(1.25f, 0.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialiv(FFP_SHININESS, high_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, no_mat);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in second row, fourth column
		 *  ambient and diffuse reflection; emission; no specular
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(3.75f, 0.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, no_mat);
		ffp.ffpMaterialiv(FFP_SHININESS, no_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, mat_emission);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in third row, first column
		 *  colored ambient and diffuse reflection; no specular
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-3.75f, -3.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient_color);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, no_mat);
		ffp.ffpMaterialiv(FFP_SHININESS, no_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, no_mat);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in third row, second column
		 *  colored ambient, diffuse and specular reflection; low shininess
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(-1.25f, -3.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient_color);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialiv(FFP_SHININESS, low_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, no_mat);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in third row, third column
		 *  colored ambient, diffuse and specular reflection; high shininess
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(1.25f, -3.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient_color);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, mat_specular);
		ffp.ffpMaterialiv(FFP_SHININESS, high_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, no_mat);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();

		/*
		 *  draw sphere in third row, fourth column
		 *  colored ambient and diffuse reflection; emission; no specular
		 */
		ffp.ffpPushMatrix();
		ffp.ffpTranslatef(3.75f, -3.0f, 0.0f);
		ffp.ffpMaterialiv(FFP_AMBIENT, mat_ambient_color);
		ffp.ffpMaterialiv(FFP_DIFFUSE, mat_diffuse);
		ffp.ffpMaterialiv(FFP_SPECULAR, no_mat);
		ffp.ffpMaterialiv(FFP_SHININESS, no_shininess);
		ffp.ffpMaterialiv(FFP_EMISSION, mat_emission);
		aux.auxSolidSphere(1.0f);
		ffp.ffpPopMatrix();
		ffp.ffpFlush(FFP_MULTI_THREADED);
	}

}
